/*
 * Copyright (C) 2023 fleroviux
 *
 * Licensed under GPLv3 or any later version.
 * Refer to the included LICENSE file.
 */

#pragma once

#include <nba/integer.hpp>
#include <string.h>

namespace nba {

template<typename T>
inline T read(void const* data, uint offset) {
  T value;
  std::memcpy(&value, (u8*)data + offset, sizeof(T));
  return value;
}

template<>
inline u8 read(void const* data, uint offset){
  u8 const *p = (u8*)data;
  return p[offset];
}

template<>
inline u16 read(void const* data, uint offset){
  u16 const *p = (u16*)data;
  return p[ offset>>1 ];
}

template<>
inline u32 read(void const* data, uint offset) {
  u32 const *p = (u32*)data;
  return p[ offset>>2 ];
}

template<typename T>
inline void write(void* data, uint offset, T value) {
  std::memcpy((u8*)data + offset, &value, sizeof(T));
}

} // namespace nba
